-- This script corrects distortion of models generated
-- by 3D Ripper DX (http://www.deep-shadows.com/hax/3DRipperDX.htm)
-- which occurs when the FOV is estimated incorrectly by the user.

-- author: Dusan Pavlicek, pavlicd@volny.cz, October 2007


fn GetSelectedVertices = (
	selectedVertices = #()
	selObjects = selection as array
	for i = 1 to selObjects.count do (
		object = selObjects[i]
		if classof object == Editable_Mesh then (
			selVertexIDs = getVertSelection object as array
			for j = 1 to selVertexIDs.count do (
				vertex = getVert object selVertexIDs[j]
				append selectedVertices vertex
			)			
		)
	)
	return selectedVertices
)


fn CalcAxisScale axisID plane1 plane2 = (
	if plane1.count != 3 then (
		MessageBox("Plane 1 is undefined.")
		return undefined
	)
	if plane2.count != 3 then (
		MessageBox("Plane 2 is undefined.")
		return undefined
	)
	p1 = plane1[1]
	p2 = plane1[2]
	p3 = plane1[3]
	q1 = plane2[1]
	q2 = plane2[2]
	q3 = plane2[3]
	a = point3 (p2.x - p1.x) (p2.y - p1.y) (p2.z - p1.z)
	b = point3 (p3.x - p1.x) (p3.y - p1.y) (p3.z - p1.z)
	c = point3 (q2.x - q1.x) (q2.y - q1.y) (q2.z - q1.z)
	d = point3 (q3.x - q1.x) (q3.y - q1.y) (q3.z - q1.z)
	u = (a.y * b.z - a.z * b.y) * (c.y * d.z - c.z * d.y)
	v = (a.z * b.x - a.x * b.z) * (c.z * d.x - c.x * d.z)
	w = (a.x * b.y - a.y * b.x) * (c.x * d.y - c.y * d.x)

	axisName = "?"
	t = -1.0
	scale = 1.0
	scaleX = 1.0
	scaleY = 1.0
	scaleZ = 1.0
	case axisID of (
		1: (
			axisName = "X"
			if v + w != 0 then (
				t = -u / (v + w)
				scale = sqrt(t)
				scaleX = scale
			)
		)
		2: (
			axisName = "Y"
			if u + w != 0 then (
				t = -v / (u + w)
				scale = sqrt(t)
				scaleY = scale
			)
		)
		3: (
			axisName = "Z"
			if u + v != 0 then (
				t = -w / (u + v)
				scale = sqrt(t)
				scaleZ = scale
			)
		)
	)

	if t > 0 then (
		MessageBox("Scaling the model by " + (scale as string) + " along the " + axisName + " axis.")
		return (point3 scaleX scaleY scaleZ)
	)
	
	if abs(u + v + w) < 0.0001 then (
		MessageBox("The two planes are already perpendicular.")
	)
	else (
		MessageBox("Sorry, the two planes can't be made perpendicular.")
	)
	return undefined
)


fn ScaleObjects scaleXYZ = (
	for object in rootNode.children do (
		if classof object == Editable_Mesh do (
			scale object scaleXYZ
		)
	)
)


rollout RolloutMain "" (
	local plane1 = #()
	local plane2 = #()

	label label1a "Define 2 planes that are supposed"
	label label1b "to be perpendicular:"
	label label1c " "

	label label2a "1) Select 3 vertices to define Plane 1"
	label label2b "and press the button."
	button buttonDefinePlane1 "Define Plane 1"

	label label3a "2) Select 3 vertices to define Plane 2"
	label label3b "and press the button."
	button buttonDefinePlane2 "Define Plane 2"

	label label4a " "
	label label4b "Then press one of the buttons to rescale"
	label label4c "all meshes along the X, Y or Z axis."

	button buttonCorrectXScale "Correct X Scale"
	button buttonCorrectYScale "Correct Y Scale"
	button buttonCorrectZScale "Correct Z Scale"
	
	on buttonDefinePlane1 pressed do (
		plane1 = GetSelectedVertices()
		if plane1.count != 3 then (
			MessageBox("Select exactly 3 vertices please.")
		)
	)

	on buttonDefinePlane2 pressed do (
		plane2 = GetSelectedVertices()
		if plane2.count != 3 then (
			MessageBox("Select exactly 3 vertices please.")
		)
	)
	
	on buttonCorrectXScale pressed do (
		scaleXYZ = CalcAxisScale 1 plane1 plane2
		if scaleXYZ != undefined then (
			ScaleObjects scaleXYZ
		)
	)
	
	on buttonCorrectYScale pressed do (
		scaleXYZ = CalcAxisScale 2 plane1 plane2
		if scaleXYZ != undefined then (
			ScaleObjects scaleXYZ
		)
	)
	
	on buttonCorrectZScale pressed do (
		scaleXYZ = CalcAxisScale 3 plane1 plane2
		if scaleXYZ != undefined then (
			ScaleObjects scaleXYZ
		)
	)
)


rollout RolloutAbout "About" (
	label label1 "Dusan Pavlicek"
	label label2 "pavlicd@volny.cz"
	label label3 "October 2007"
)


if MyFloater != undefined do (
	closerolloutfloater MyFloater
)
MyFloater = newRolloutFloater "Correct XYZ Scale" 260 500
addRollout RolloutMain MyFloater
addRollout RolloutAbout MyFloater
